/*
 * z_utl.c: Utility functions
 * Copyright (C) 2014  Matthias Zartmann.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 * USA
 *
 * Authors:
 *     Matthias Zartmann <matthias.zartmann@gmx.de>
 * Created on: 4 Apr 2014
 */
#include "z_util.h"


/**
 * Gets a Booleanvalue from a Keyfile
 * @param key_file the Keyfile instance
 * @param group_name The Group [Section] name
 * @param key the Key in the section
 * @param b_default the Default value if the Key not exists
 * @return the Value
 */
gboolean z_util_key_file_get_boolean(GKeyFile *key_file,const gchar *group_name,const gchar *key, gboolean b_default)
{
	gboolean res;
	GError *gerror =NULL;

	res = g_key_file_get_boolean(key_file,group_name,key,&gerror);

	if (res == FALSE && gerror != NULL)
	{
		z_critical_if_fail(! (gerror->code == G_KEY_FILE_ERROR_INVALID_VALUE),
				               "[%s] %s can't be parsed as boolean",group_name,key);
		res = b_default;
	}
  return res;
}

/**
 * Gets a string value from a Keyfile
 * @param key_file the Keyfile instance
 * @param group_name The Group [Section] name
 * @param key the Key in the section
 * @param s_default the Default value if the Key not exists
 * @return the Value (use G_free)
 */
gchar * z_util_key_file_get_string(GKeyFile *key_file,const gchar *group_name,const gchar *key, gchar * s_default)
{
	gchar * res;
	GError *gerror =NULL;

	res = g_key_file_get_string(key_file,group_name,key,&gerror);
	if (res == NULL && gerror != NULL)
	{
		z_critical_if_fail(! (gerror->code == G_KEY_FILE_ERROR_INVALID_VALUE),
				               "[%s] %s can't be parsed as String",group_name,key);
		res = s_default;
	}
  return res;
}
/**
 * Gets a int value from a Keyfile
 * @param key_file the Keyfile instance
 * @param group_name The Group [Section] name
 * @param key the Key in the section
 * @param s_default the Default value if the Key not exists
 * @return the Value (use G_free)
 */
gint z_util_key_file_get_int(GKeyFile *key_file,const gchar *group_name,const gchar *key, gint i_default)
{
	gint res;
	GError *gerror =NULL;

	res = g_key_file_get_integer(key_file,group_name,key,&gerror);
	if (res == 0 && gerror != NULL)
	{
		z_critical_if_fail(! (gerror->code == G_KEY_FILE_ERROR_INVALID_VALUE),
				               "[%s] %s can't be parsed as integer",group_name,key);


		res = i_default;
	}
  return res;
}

/**
 * Check if file exist
 * @param pfilename  the filename
 * @return TRUE the file exists
 */
gboolean z_util_file_exists(gchar* pfilename)
{
	GFile * file = g_file_new_for_path(pfilename);
	gboolean res = FALSE;
	if (g_file_query_exists(file, NULL))
		res = TRUE;
	g_object_unref(G_OBJECT(file));
 return res;
}


/**
 * process all GTK messages
 */
void z_util_process_messages(void)
{
  while (gtk_events_pending ())
   gtk_main_iteration ();

}
