/*
 * z_cfg.c: Read Write configuration
 * first read  [exe-path]/zradio.cfg
 * Override by /etc/zradio.cfg
 * Override by ~/.config/zradio.cfg
 *
 * Copyright (C) 2014  Matthias Zartmann.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 * USA
 *
 * Authors:
 *     Matthias Zartmann <matthias.zartmann@gmx.de>
 * Created on: 4 Apr 2014
 */
#include "zr_cfg.h"
#include "z_util.h"

/**
 * Internal function to free a station node
 * @param station
 */
static void zr_cfg_station_destroy(zr_cfg_station * station)
{
	g_free(station->key);
	g_free(station->name);
	g_free(station->url);
}

/**
 * Internal add a new station (override duplicates)
 * @param p_cfg the Configuration
 * @param station the station to add
 */
static void zr_cfg_addStation(zr_cfg * p_cfg, zr_cfg_station * station)
{
	zr_cfg_station * element;
	gint i = 0;
	gint found = -1;
	while ((element = &g_array_index(p_cfg->stations, zr_cfg_station, i)) &&(element->key!= NULL))
	{
		if (g_strcmp0(element->key, station->key) == 0)
		{
			found = i;
			break;
		}
		i++;
	}

	if (found > -1)
	{
		element = &g_array_index(p_cfg->stations, zr_cfg_station, found);
		zr_cfg_station_destroy(element);
		element->key = station->key;
		element->name = station->name;
		element->url = station->url;
	} else
	{
		g_array_append_val(p_cfg->stations, * station);
	}
}

/**
 * Load a Configuration
 * @param p_cfg the Config instance
 * @param filename the filename to load
 */
void _zr_cfg_load(zr_cfg * p_cfg, gchar * filename)
{

	if (z_util_file_exists(filename))
	{
		g_debug("Load configfile %s",filename );

		GKeyFile * inifile = g_key_file_new();
		GError *gerror = NULL;
		z_error_if_fail(g_key_file_load_from_file(inifile, filename, G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, &gerror)
				            ,"g_key_file_load_from_file fail: %s",  gerror->message );


		//Load configvalues
		if (g_key_file_has_group(inifile, ZR_CFG_SEC_CONFIG))
		{
			p_cfg->play_on_start = z_util_key_file_get_boolean(inifile, ZR_CFG_SEC_CONFIG, ZR_CFG_KEY_PLAY_ON_START, FALSE);
			p_cfg->lasturl = z_util_key_file_get_string(inifile, ZR_CFG_SEC_CONFIG, ZR_CFG_KEY_LASTURL, NULL);
			p_cfg->volume = z_util_key_file_get_int(inifile, ZR_CFG_SEC_CONFIG, ZR_CFG_KEY_VOLUME, 80);
		}

		//Load Stationlist
		if (g_key_file_has_group(inifile, ZR_CFG_SEC_STATION))
		{
			gchar ** keys;
			gsize c = 0;
			gerror = NULL;
			keys = g_key_file_get_keys(inifile, ZR_CFG_SEC_STATION, &c, &gerror);
			if (c > 0)
			{
				int i = 0;
				gsize list_length = 0;
				gchar ** values;
				gerror = NULL;
				zr_cfg_station station;
				for (i = 0; i < c; i++)
				{
					values = g_key_file_get_string_list(inifile, ZR_CFG_SEC_STATION, keys[i], &list_length, &gerror);

					if (list_length == 2)
					{
						station.key = g_strdup(keys[i]);
						station.name = g_strdup(values[0]);
						station.url = g_strdup(values[1]);
						zr_cfg_addStation(p_cfg, &station);
					} else
					{
						g_critical("Wrong config entry %s. Syntax must be key=Name;URL", keys[i]);
					}
					g_strfreev(values);
				}
			}
			g_strfreev(keys);
		}
		g_key_file_free(inifile);
	} else
	{
		g_message("skip configfile  %s", filename);
	}

}
/*----------------------------------------------------------------------------------------------------------------*/

/**
 * Create a new Config Instance
 * @param p_exepath Path to the ZRadio Exe
 * @return the instance
 */
zr_cfg * zr_cfg_create(gchar * p_exepath)
{
	zr_cfg * result = g_new0(zr_cfg, 1);
	result->stations = g_array_new(TRUE, TRUE, sizeof(zr_cfg_station));
	g_array_set_clear_func(result->stations, (GDestroyNotify) zr_cfg_station_destroy);
	gchar * config;

	result->exepath =  g_strdup(p_exepath);

	//Loading form exe dir
	config = g_strdup_printf("%s/zradio.cfg", p_exepath);
	_zr_cfg_load(result, config);
	g_free(config);

	//Loading form etc
	_zr_cfg_load(result, "/etc/zradio.cfg");


	//Loading form etc
    _zr_cfg_load(result, "/usr/local/etc/zradio.cfg");



	//Loading form ~/.config dir
	config = g_strdup_printf("%s/zradio.cfg", g_get_user_config_dir());
	_zr_cfg_load(result, config);
	g_free(config);

	return result;
}


/**
 * Free the Configuration
 * @param p_cfg
 */
void zr_cfg_free(zr_cfg * p_cfg)
{
	g_free(p_cfg->lasturl);
	g_free(p_cfg->exepath);
	g_array_free(p_cfg->stations, TRUE);

	g_free(p_cfg);

}
/**
 * Start Writing to the User-Configuration
 * @return the GKeyFile instance
 */
GKeyFile *  zr_cfg_begin_write()
{
	gchar * config = g_strdup_printf("%s/zradio.cfg", g_get_user_config_dir());
	GKeyFile * inifile = g_key_file_new();
	GError *gerror = NULL;
	z_error_if_fail(g_key_file_load_from_file(inifile, config, G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS, &gerror)
			            ,"g_key_file_load_from_file fail: %s"
			            , gerror->message);
	g_free(config);
	return inifile;
}

/**
 * End of Writing to the User-Configuration
 * @param p_inifile
 */
void zr_cfg_end_write(GKeyFile * p_inifile)
{
	gsize len;
	GError *gerror = NULL;
	gchar * configfile = g_strdup_printf("%s/zradio.cfg", g_get_user_config_dir());

	gchar * buffer =  g_key_file_to_data(p_inifile,&len,&gerror);
  if (buffer != NULL)
  {
  	GFile *file = g_file_new_for_path(configfile);
  	gerror = NULL;
  	gboolean res = g_file_replace_contents (file,buffer,len,NULL,FALSE,G_FILE_CREATE_NONE,NULL,NULL,&gerror);
  	if (!res)
  	{
  		g_critical("Can't save user configfile %s. %s",configfile, gerror->message);
  	}
    g_object_unref(G_OBJECT(file));
  }
  g_free(buffer);
  g_free(configfile);
  g_key_file_free(p_inifile);
}

/**
 * Writing a Value to the User-Configuration
 * @param p_inifile the GKeyFile from zr_cfg_begin_write
 * @param section Sectionname
 * @param key Key Name
 * @param value the Value to write
 */
void zr_cfg_write_value(GKeyFile * p_inifile, const gchar * section, const gchar * key, const gchar * value)
{
	g_key_file_set_value(p_inifile, section, key, value);

}

