/*
 * z_gst.c: Interface to the gstreamer
 *
 * Copyright (C) 2014  Matthias Zartmann.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 * USA
 *
 * Authors:
 *     Matthias Zartmann <matthias.zartmann@gmx.de>
 * Created on: 4 Apr 2014
 */
#include "zr_standard.h"

#include "zr_app.h"
#include "zr_gst.h"


/**
 * Parsing Gstreamer Tags
 * @param list Gstreamer GstTagList
 * @param tag the tag
 * @param p_gst Instance of Gstreamer
 */
void zr_tag_emit(const GstTagList *list, const gchar *tag, zr_gst * p_gst)
{
	const GValue * tvalue;

	tvalue = gst_tag_list_get_value_index(list, tag, 0);
	if (tvalue == NULL)
	{
		return;
	}
	if (G_VALUE_HOLDS_STRING (tvalue) == FALSE)
	{
		return;
	}
	if (g_strcmp0(tag, GST_TAG_TITLE) == 0)
	{
		if (g_strcmp0(p_gst->CurrentSong,  g_value_get_string(tvalue) ) != 0) //Song must change to old one (void double signal )
		{
  		g_strlcpy(p_gst->CurrentSong, g_value_get_string(tvalue), sizeof(p_gst->CurrentSong));
	  	zr_app_emit_event(ZR_APP_MSG_ICY_CHANGE, "SONG");
		}
	} else if (g_strcmp0(tag, GST_TAG_LOCATION) == 0)
	{
		g_strlcpy(p_gst->CurrentURL, g_value_get_string(tvalue), sizeof(p_gst->CurrentURL));
		zr_app_emit_event(ZR_APP_MSG_ICY_CHANGE, "ICY_URL");
	} else if (g_strcmp0(tag, GST_TAG_ORGANIZATION) == 0)
	{
		g_strlcpy(p_gst->CurrentNAME, g_value_get_string(tvalue), sizeof(p_gst->CurrentNAME));
		zr_app_emit_event(ZR_APP_MSG_ICY_CHANGE, "ICY_NAME");
	}

}

/**
 * The Gstreamer bus message Handler
 * @param bus The Gstreamer Bus
 * @param message the Message
 * @param p_gst Gstreamer instance
 * @return FALSE if the event source should be removed.
 */
gboolean zr_gst_BusMessages(GstBus *bus, GstMessage *message, zr_gst * p_gst)
{

//	g_debug("Source=%s type=%d", GST_MESSAGE_SRC_NAME(message), GST_MESSAGE_TYPE(message));

	switch (GST_MESSAGE_TYPE(message))
	{
		case GST_MESSAGE_EOS:
			g_debug("GST_MESSAGE_EOS");
			break;

		case GST_MESSAGE_DURATION:
		{
			gint64 duration;
			GstFormat format = GST_FORMAT_BYTES;

			gst_message_parse_duration(message,&format,&duration);
			if (duration > 500)
   			p_gst->errorcount = 0; //Stream runs
		}
		break;


		case GST_MESSAGE_ERROR:
		{
			GError *err = NULL;
			gchar *dbg_info = NULL;
			gst_message_parse_error(message, &err, &dbg_info);
			g_printerr("ERROR from element %s: %s Code=%d\n", GST_OBJECT_NAME(message->src), err->message, err->code);
			g_printerr("Debugging info: %s\n", (dbg_info) ? dbg_info : "none");
			//No infinitv in an error loop
			if ( p_gst->errorcount < 1)
			{
				p_gst->errorcount ++;
			  zr_app_emit_event(ZR_APP_MSG_STREAM_ERROR,  err->message);
			}
			g_error_free(err);
			g_free(dbg_info);
   		g_debug("GST_MESSAGE_ERROR");


			break;
		}
		case GST_MESSAGE_STREAM_STATUS:
		{
			GstStreamStatusType status;
			GstElement *owner;
			gst_message_parse_stream_status(message, &status, &owner);
			p_gst->currentStatus = status;
			switch (status)
			{
				case 0:
					g_debug("Creating Stream\n");
					break;
				case 1:
					g_debug("Entering Stream\n");
					break;
				case 2:
					g_debug("Leaving Stream\n");
					break;
				case 3:
					g_debug("Destorying Stream\n");
					break;
				case 8:
					g_debug("Staring Stream\n");
					break;
				case 9:
					g_debug("Pausing Stream\n");
					break;
				case 10:
					g_debug("Stoping Stream\n");
					break;
				default:
					break;
			}

			break;
		}
		case GST_MESSAGE_TAG:
		{
			GstTagList *tags = NULL;
			gst_message_parse_tag(message, &tags);
			gst_tag_list_foreach(tags, (GstTagForeachFunc) zr_tag_emit, p_gst);
			gst_tag_list_free(tags);
			break;
		}
		case GST_MESSAGE_CLOCK_LOST:
			/* Get a new clock */
			gst_element_set_state(p_gst->pipeline, GST_STATE_PAUSED);
			gst_element_set_state(p_gst->pipeline, GST_STATE_PLAYING);
			break;

		case GST_MESSAGE_STATE_CHANGED:
		{
			GstState old_state, new_state, pending_state;
			gst_message_parse_state_changed(message, &old_state, &new_state, &pending_state);
			if (GST_MESSAGE_SRC (message) == GST_OBJECT(p_gst->pipeline))
			{
				g_debug("Pipeline state changed from %s to %s:\n", gst_element_state_get_name(old_state), gst_element_state_get_name(new_state));
				p_gst->currentState =new_state;

			}

		}
			break;

		case GST_MESSAGE_BUFFERING:
		{
			gint percent = 0;
			gst_message_parse_buffering(message, &percent);
			g_debug("Cache:%d ", percent);
      if (percent > 10)
      {
      	p_gst ->start_watch_bufferunderrun = TRUE;
      }


			if ((percent == 0) && (p_gst ->start_watch_bufferunderrun) )
			{

				zr_app_emit_event(ZR_APP_MSG_BUFFER_EMPTY, NULL);
			}

		}
			break;
		default:
			break;
	}

	return TRUE;
}

/**
 * Internal initialising
 * @param p_gst instance Gstreamer
 */
static void zr_init(zr_gst * p_gst)
{
	GError * error = NULL;
	if (!gst_init_check(NULL, NULL, &error))
	{
		g_error("Can't initalize GStreamer.  %s  ", error->message);
	}

	p_gst->pipeline = gst_element_factory_make("playbin2", "player");
	if (p_gst->pipeline == NULL)
	{
		g_error("gst_element_factory_make(playbin,Player) fails");
	}

	GstBus *bus = gst_pipeline_get_bus(GST_PIPELINE(p_gst->pipeline));
	gst_bus_add_watch(bus, (GstBusFunc) zr_gst_BusMessages, p_gst);
	gst_object_unref(bus);
}

/**
 * Internal free
 * @param p_gst instance Gstreamer
 */
static void zr_free(zr_gst * p_gst)
{
	gst_element_set_state(p_gst->pipeline, GST_STATE_NULL);
	gst_object_unref(p_gst->pipeline);
	gst_deinit();
}

/**
 * Create new Gstreamer Instance
 * @return Gstreamer Instance
 */
zr_gst * zr_gst_create()
{

	zr_gst * result = g_new0(zr_gst, 1);
	zr_init(result);
	return result;
}


/**
 * Free Gstreamer Instance
 * @param p_gst Gstreamer Instance
 */
void zr_gst_free(zr_gst * p_gst)
{
	zr_free(p_gst);

}

/**
 * Play Url
 * @param p_gst Gstreamer Instance
 * @param url the URL
 */
void zr_gst_play(zr_gst * p_gst, gchar * url)
{

	p_gst ->start_watch_bufferunderrun = FALSE;
	g_debug("Play %s ", url);
	gst_element_set_state(GST_ELEMENT(p_gst->pipeline), GST_STATE_NULL);
	g_object_set(G_OBJECT(p_gst->pipeline), "uri", url, NULL);
	g_strlcpy(p_gst->CurrentPlayingURL, url, sizeof(p_gst->CurrentPlayingURL));

	gst_element_set_state(GST_ELEMENT(p_gst->pipeline), GST_STATE_PLAYING);
}

/**
 * Get the current State of the Gstreamer
 * @param p_gst Gstreamer instance
 * @return GstState
 */
static GstState getState(zr_gst * p_gst)
{
	GstState state;
	GstStateChangeReturn res;
	res = gst_element_get_state(GST_ELEMENT(p_gst->pipeline), &state, NULL, GST_CLOCK_TIME_NONE);
	if (res == GST_STATE_CHANGE_SUCCESS)
	{
		return state;
	} else
	{
		return GST_STATE_NULL;
	}

}
/**
 * Pause current playing
 * @param p_gst Gstreamer instance
 */
void zr_gst_pause(zr_gst * p_gst)
{
	GstState state = getState(p_gst);
	if (state == GST_STATE_NULL)
		gst_element_set_state(GST_ELEMENT(p_gst->pipeline), GST_STATE_PLAYING);
	else
		gst_element_set_state(GST_ELEMENT(p_gst->pipeline), GST_STATE_NULL);
}

/**
 * Set the Outputvolume
 * @param p_gst Gstreamer instance
 * @param p_gst volume 0-100
 */
void zr_gst_set_volume(zr_gst * p_gst, int volume)
{
	gst_stream_volume_set_volume(GST_STREAM_VOLUME(p_gst->pipeline), GST_STREAM_VOLUME_FORMAT_LINEAR, (gdouble) (volume / 100.0));
}

/**
 * Get the current Info
 * @param p_gst  Gstreamer instance
 * @param p_info Pointer to zr_gst_info
 */
void zr_gst_get_info(zr_gst * p_gst, zr_gst_info * p_info)
{
	GstState state = getState(p_gst);
	p_info->playing = (state == GST_STATE_PLAYING);

	p_info->icy_Song = p_gst->CurrentSong;
	p_info->currentplaying_url = p_gst->CurrentPlayingURL;
	p_info->icy_station = p_gst->CurrentNAME;
	p_info->volume = gst_stream_volume_get_volume(GST_STREAM_VOLUME(p_gst->pipeline), GST_STREAM_VOLUME_FORMAT_LINEAR) * 100;

}


/**
 * Reset the Gstreamer
 * @param p_gst Gstreamer instance
 * @param url the URL to Play
 */
void zr_gst_reset(zr_gst * p_gst,gchar * url)
{
	g_debug("zr_gst_reset");

	gst_element_set_state(p_gst->pipeline, GST_STATE_NULL);
	zr_gst_play(p_gst,url);

}
/**
 * Check if gGstreamer current playing
 * @param p_gst  Gstreamer instance
 * @return TRUE = Playing
 */
gboolean zr_gst_is_playing(zr_gst * p_gst)
{
	return p_gst->currentState == GST_STATE_PLAYING;

}
